package fishProject;

//import javax.swing.UIManager;
//import java.awt.*;
//import java.awt.event.*;
//import javax.swing.*;
import java.lang.*;
import java.util.*;
import java.io.*;

import gl4java.GLContext;
import gl4java.awt.GLAnimCanvas;
import gl4java.utils.textures.*;
import gl4java.utils.glut.*;

/***********************************************************************************
 * Title:      	FishGLCanvas
 * Description:	The canvas class. This is were all the graphical happens.
 * @author 		Bjrn Hkon Horpestad
 * @version 	1.0
************************************************************************************/

public class FishGLCanvas extends GLAnimCanvas {
		GLUTFunc glut = null;
		private Fish myFish;
		private BezierCurve bezCurve;
		private Vector3D T, B, C, N;
		private double dt = 0.1; // seconds between frames.
		private int framecount = 0;
		private float newZ = 0.0f;
		private boolean cWise = true;
		private boolean fishAnimate, fishSwim, fogOn;
		private int showix = 1;
		private int stepix = 1;
		private int bodyTexture, eyeTexture, finTexture;
		private int[] texture; //Storage for textures
		
		// variables for rotation of the fish.
		private float rotx = 0.0f;
		private float rotz = 0.0f;
		private float lastx = 0.0f;
		private float lastz = 0.0f;
		private float delta_rotx = 1.0f;
		private float delta_rotz = 1.0f;
		
		/**
		*	constructor
		*	This is the constructor.
		*	@param w - int that gives the width of the canvas.
		*	@param h - int that gives the height of the canvas.
		**/
		public FishGLCanvas(int w, int h) { 
			super(w,h); 	
			myFish = new Fish(1.0f);
			bezCurve = new BezierCurve(350);
			setBodyTexture(9);
			setFinTexture(10);
			setEyeTexture(11);
			setFishAnimate(false);
			setFishSwim(false);
			setFogOn(false);
		}
			
		/**
		*	preInit()
		*	Method that does some thing before init.
		*
		**/
		public void preInit() {
			doubleBuffer = true;
			stereoView = false;
			accumSize = 8;	
		}
		
		/**
		*	init()
		*	Method that sets up and starts the canvas.
		*
		**/
		public void init() {
			glut = new GLUTFuncLightImpl(gl, glu);
			
			// the textures files that are used by the program
			String[] files = { "fishProject\\pictures\\fishTexture_6.png", 
							   "fishProject\\pictures\\fishFinTexture_6.png", 
							   "fishProject\\pictures\\fish_eye_6.png",
							   "fishProject\\pictures\\fishTexture_4.png", 
							   "fishProject\\pictures\\fishFinTexture_4.png", 
							   "fishProject\\pictures\\fish_eye_4.png",
							   "fishProject\\pictures\\fishTexture_5.png", 
							   "fishProject\\pictures\\fishFinTexture_5.png", 
							   "fishProject\\pictures\\fish_eye_5.png",
							   "fishProject\\pictures\\fishTexture_10.png",
							   "fishProject\\pictures\\fishFinTexture_15.png", 
							   "fishProject\\pictures\\fish_eye_9.png",
							   "fishProject\\pictures\\fishTexture_12.png",
							   "fishProject\\pictures\\fish_eye_11.png"
							   };
			// load all the files.
			loadGLTexture(files);
			
			reshape(getSize().width, getSize().height);
			// sets the light
			setLight();			
			//smooth the drawing
			gl.glShadeModel(GL_SMOOTH);
			//normalize normals
			gl.glEnable(GL_NORMALIZE);
			//depth sorting
			gl.glEnable(GL_DEPTH_TEST);
			gl.glDepthFunc(GL_LESS);
			//set background
			//gl.glClearColor(1.0f, 1.0f, 1.0f, 0.0f); //White
			gl.glClearColor(0.5f, 0.6f, 1.0f, 0.0f); //Light blue
			// set initial animation speed.
			setAnimateFps(1.0/dt);
		}
		
		/**
		*	display()
		*	Method that displays all openGL things.
		*	
		**/
		public void display() {
			if(glj.gljMakeCurrent() == false)
				return;
			
			gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			gl.glClear(GL_ACCUM_BUFFER_BIT);
			gl.glLoadIdentity();			
			
			// Decides if the fog should be on or off.
			if(fogOn)
				makeFog();
			else
				gl.glDisable(GL_FOG);
			
			showScene();
			
			
			 /***********
			 ** Increments here GLAnimCanvas is it's own engine
			 ** Calculate new z positions to be used.
			 **	Only does this if fishAnimate is true.
			 ***********/
			
			if(fishAnimate) {
				if(newZ > myFish.getMaxSwing())
					cWise = false;
				if(newZ < myFish.getMinSwing())
					cWise = true;
				if(cWise) {
					newZ = newZ + 0.1f;
				} else {
					newZ = newZ - 0.1f;
				}
				myFish.moveFish(newZ);	
				myFish.getFishTail().moveTail(newZ);
				myFish.getBackFin().moveBackFin(newZ);	
				myFish.getLeftFin().moveSideFin(newZ);	 
				myFish.getRightFin().moveSideFin(newZ);
			}
			
			/*****************************
			**
			**	The increment of showix. 
			**	showix indicates the position to the fish on the BezierCurve.
			**	Only done if the boolean variable fishSwim is true.
			**
			*******************************/
			if(fishSwim) {
				if(showix==bezCurve.getOrder())
					stepix=-1;
				else if(showix==0)
					stepix=1;
				showix += stepix;
			 }
			glj.gljSwap();
			glj.gljFree();	
		}
		
		/**
		*	reshape()
		*	Method that sees to that everything is updated if the size to the application is changed.
		*	@param w - int that gives the new width of the application
		*	@param h - int that gives the new height of the application
		**/
		public void reshape(int w, int h) {
			gl.glViewport(0, 0 , w, h);
			gl.glMatrixMode(GL_PROJECTION);
			gl.glLoadIdentity();
			glu.gluPerspective(45.0, (float) w/h, 1.0, 500.0);
			gl.glMatrixMode(GL_MODELVIEW);	
		}
		
		/**
		*	showScene()
		*	Method that sets up lookAt.
		*	It is also the most important method in the use of frenet frames. 
		*
		*
		**/
		public void showScene(){
			
			glu.gluLookAt(	0.0f,0.0f,40.0f,	// eye
							0.0f,0.0f,0.0f,	// looking at
							0.0f,1.0f,0.0f);// is up
							
			// The calculation of the bezier Curve that is used when the fish swims across the screen.
			// Only done if the boolean value fishSwim is true.				
			if(fishSwim) {
				int ix=0;
				while(ix < bezCurve.getOrder()) {
		
				// Bezier is now tabulated in:
				//	(bez[ix][0],bez[ix][1],bez[ix][2])
				// The derivative (tangent) is tabulated in:
				//	(d_bez[ix][0],d_bez[ix][1],d_bez[ix][2])
				// The double derivated is tabulated in:
				//	(dd_bez[ix][0],dd_bez[ix][1],dd_bez[ix][2])

				// finding the normalized tangent: 
				// T pointing along the curve
				T = new Vector3D(bezCurve.getDBezAtPos(ix, 0), bezCurve.getDBezAtPos(ix, 1),bezCurve.getDBezAtPos(ix, 2));
				T.normalize();

				// finding the normalized d cross dd vector: 
				// B pointing perpendicular to T
				B = new Vector3D(bezCurve.getDBezAtPos(ix, 0), bezCurve.getDBezAtPos(ix, 1),bezCurve.getDBezAtPos(ix, 2));
				//B.cross(bezCurve.getDDBezAtPos(ix, 0), bezCurve.getDDBezAtPos(ix, 1),bezCurve.getDDBezAtPos(ix, 2));
				B.cross(0.0f,0.0f,1.0f);
				B.normalize();
	
				// finding a third perpendiculat vector: 
				// N perpendicular to both T and B
				N = new Vector3D(B.getX(),B.getY(),B.getZ());
				N.cross(T);  // and it is normalized since B and T are

				// we want to move to the correct point on the Beziercurve
				// as tabulated in (bez[ix][0],bez[ix][1],bez[ix][2])
				C = new Vector3D(bezCurve.getBezAtPos(ix, 0), bezCurve.getBezAtPos(ix, 1),bezCurve.getBezAtPos(ix, 2));
		
				// at this point T,B,N describes the coordinate system
				// we will use at the position C 
				// N will act as x-axes
				// B will act as y-axes
				// T will act as z-axes

				// setting up the matix that will take us to the
				// Frenet frame located at this t-point
				// M=|N,B,T,C|, where C is the Bezier itself
				// note transpose of matrix

				float M[]={
				N.getX(),N.getY(),N.getZ(),0.0f,
				B.getX(),B.getY(),B.getZ(),0.0f,
				T.getX(),T.getY(),T.getZ(),0.0f,
				C.getX(),C.getY(),C.getZ(),1.0f
				};
		
				// take care of the matrix we have
				gl.glPushMatrix();
		
				//multiply our newly made Frenet matrix to OpenGLs current matrix
				gl.glMultMatrixf(M);

			
				/*****************************
				**
				** Vectors and stuff done, we are ready to draw.
				** Draws the fish at the showix position.
				**
				*********************************/
				if( showix == ix) {
					gl.glRotatef(90, 0.0f, 1.0f, 0.0f);
					gl.glRotatef(180, 0.0f, 0.0f, 1.0f);
					if(stepix == -1) {
						gl.glRotatef(180, 0.0f, 1.0f, 0.0f);
						makeFish();
					} else 
						makeFish();
				}

				gl.glPopMatrix();
			
				ix++;
				}
			}
			else 
				makeFish();
			gl.glFlush();	
		}
		
		/**
		*	makeFish()
		*	Method that makes the fish. 
		*	It mostly redirect the responsibilities to other methods.
		*
		**/
		public void makeFish() {
			gl.glPushMatrix();
			gl.glEnable(GL_TEXTURE_2D);
			drawFishBody();
			gl.glDisable(GL_TEXTURE_2D);
			gl.glPushMatrix();
			gl.glEnable(GL_TEXTURE_2D);
			drawTail();
			gl.glDisable(GL_TEXTURE_2D);	
			gl.glPopMatrix();
			gl.glPushMatrix();
			gl.glEnable(GL_TEXTURE_2D);
			drawBackFin();
			gl.glDisable(GL_TEXTURE_2D);
			gl.glPopMatrix();
			gl.glPushMatrix();
			gl.glEnable(GL_TEXTURE_2D);
			drawEye();
			gl.glDisable(GL_TEXTURE_2D);
			gl.glPopMatrix();
			gl.glPushMatrix();
			gl.glEnable(GL_TEXTURE_2D);
			drawLeftFin();
			gl.glDisable(GL_TEXTURE_2D);
			gl.glPopMatrix();			
			gl.glPushMatrix();
			gl.glEnable(GL_TEXTURE_2D);
			drawRightFin();
			gl.glDisable(GL_TEXTURE_2D);
			gl.glPopMatrix();
			gl.glPopMatrix();
		}
		
		/***
		*	drawFishBody()
		*	Method that adds texture to and draws the fishBody.
		*	
		*
		**/
		public void drawFishBody() {
			framecount++;
			if(framecount == 2) {
				double maxfps = getMaxFps();
				dt = 1.0/(maxfps*0.6);
				setAnimateFps(1.0/dt);
			}
			
			// if the fish is dragged rotate
			gl.glRotatef(rotx, 1.0f, 0.0f, 0.0f);
			gl.glRotatef(rotz, 0.0f, 0.0f, 1.0f);
			
		/* First, tell it where to find the control points */
  			gl.glMap2f(GL_MAP2_VERTEX_3, 0.0f, 1.0f, 3, myFish.getUN(), 0.0f, 1.0f, 3*myFish.getUN(), myFish.getVN(), myFish.getFishBody());
  			gl.glEnable(GL_MAP2_VERTEX_3);
  		/* Generate normals automatically at every vertex! */
  			gl.glEnable(GL_AUTO_NORMAL);
  		/*	Texture Mapping*/	
  			gl.glBindTexture(GL_TEXTURE_2D, texture[getBodyTexture()]);
  			gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, getTxtPoints());
  			gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
      		gl.glEnable(GL_MAP2_VERTEX_3);
		/* Draw the bezier*/
  			gl.glMapGrid2f(40, 0.0f, 1.0f, 40, 0.0f, 1.0f);
			
  			gl.glEvalMesh2(GL_FILL, 0, 40, 0, 40);				
		}
		
		/**
		*	drawTail()
		*	Method that sets texture to and draws the tail. 
		*
		**/
		public void drawTail() {
			gl.glEnable(gl.GL_MAP2_VERTEX_3);
     		gl.glEnable(gl.GL_AUTO_NORMAL);
      		gl.glEnable(gl.GL_NORMALIZE);
      		
			gl.glMap2f(GL_MAP2_VERTEX_3, 0.0f, 1.0f, 3, myFish.getFishTail().getUN(), 0.0f, 1.0f, 
														3*myFish.getFishTail().getUN(), 
														myFish.getFishTail().getVN(), myFish.getFishTail().getTailCoord());
			  		/*	Texture Mapping*/	
  			gl.glBindTexture(GL_TEXTURE_2D, texture[getFinTexture()]);
  			gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, getTxtPoints());
  			gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
      		gl.glEnable(GL_MAP2_VERTEX_3);
			gl.glMapGrid2f(20, 0.0f, 1.0f, 20, 0.0f, 1.0f);
  			gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);	
		}
		
		/**
		*	drawBackFin()
		*	Method that sets the texture on and draws the back fin.
		*
		**/
		public void drawBackFin() {

			gl.glEnable(gl.GL_MAP2_VERTEX_3);
     		gl.glEnable(gl.GL_AUTO_NORMAL);
      		gl.glEnable(gl.GL_NORMALIZE);
      		

      		gl.glRotatef(-90, 0.0f, 0.0f, 1.0f);
      		gl.glTranslatef( -0.6f *  myFish.getFactor(),0.5f * myFish.getFactor(), 0.0f);
			gl.glMap2f(GL_MAP2_VERTEX_3, 0.0f, 1.0f, 3, myFish.getBackFin().getUN(), 0.0f, 1.0f, 
														3*myFish.getBackFin().getUN(), 
														myFish.getBackFin().getVN(), myFish.getBackFin().getBackFinCoord());
			  		/*	Texture Mapping*/	
  			gl.glBindTexture(GL_TEXTURE_2D, texture[getFinTexture()]);
  			gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, getTxtPoints());
  			gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
      		gl.glEnable(GL_MAP2_VERTEX_3);
			
			gl.glMapGrid2f(20, 0.0f, 1.0f, 20, 0.0f, 1.0f);
  			gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);	
		}
		
		/**
		*	drawEye()
		*	Method that sets texture on eyes, and draws them
		*
		**/
		public void drawEye() {

			//gl.glEnable(gl.GL_MAP2_VERTEX_3);
     		gl.glEnable(gl.GL_AUTO_NORMAL);
      		gl.glEnable(gl.GL_NORMALIZE);
      		
      		gl.glTranslatef(2.6f * myFish.getFactor(), 0.2f* myFish.getFactor(), 0.2f* myFish.getFactor()); 
      		gl.glRotatef(-90, 1.0f, 0.0f, 0.0f);
			gl.glMap2f(GL_MAP2_VERTEX_3, 0.0f, 1.0f, 3, myFish.getFishEye().getUN(), 0.0f, 1.0f, 
														3*myFish.getFishEye().getUN(), 
														myFish.getFishEye().getVN(), myFish.getFishEye().getEyeCoord());
														
			gl.glBindTexture(GL_TEXTURE_2D, texture[getEyeTexture()]);														
  			gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, getTxtPoints());
  			gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
      		gl.glEnable(GL_MAP2_VERTEX_3);
														
			gl.glMapGrid2f(20, 0.0f, 1.0f, 20, 0.0f, 1.0f);
  			gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
  			gl.glRotatef(180, 1.0f, 0.0f, 0.0f);
       		gl.glTranslatef(0.0f, -0.4f * myFish.getFactor(), 0.0f);
       		gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);	
		}
		
		/**
		*	drawLeftFin()
		*	Method that sets texture on the left fin and draws it.
		*
		**/
		public void drawLeftFin() {

			gl.glEnable(gl.GL_MAP2_VERTEX_3);
     		gl.glEnable(gl.GL_AUTO_NORMAL);
      		gl.glEnable(gl.GL_NORMALIZE);
      		
      		gl.glTranslatef(0.6f * myFish.getFactor(), 0.0f, -0.4f * myFish.getFactor());
			gl.glMap2f(GL_MAP2_VERTEX_3, 0.0f, 1.0f, 3, myFish.getLeftFin().getUN(), 0.0f, 1.0f, 
														3*myFish.getLeftFin().getUN(), 
														myFish.getLeftFin().getVN(), myFish.getLeftFin().getSideFinCoord());
			  		/*	Texture Mapping*/	
  			gl.glBindTexture(GL_TEXTURE_2D, texture[getFinTexture()]);
  			gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, getTxtPoints());
  			gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
      		gl.glEnable(GL_MAP2_VERTEX_3);
			gl.glMapGrid2f(20, 0.0f, 1.0f, 20, 0.0f, 1.0f);
  			gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
		}
		
		/**
		*	drawRightFin()
		*	Method that sets texture on the right fin and draws it.
		*
		**/		
		public void drawRightFin() {

			gl.glEnable(gl.GL_MAP2_VERTEX_3);
     		gl.glEnable(gl.GL_AUTO_NORMAL);
      		gl.glEnable(gl.GL_NORMALIZE);
      		
      		
      		gl.glTranslatef(1.6f * myFish.getFactor(), 0.0f, 0.4f * myFish.getFactor());
      		//gl.glRotatef(180, 0.0f, 1.0f, 0.0f);
			gl.glMap2f(GL_MAP2_VERTEX_3, 0.0f, 1.0f, 3, myFish.getRightFin().getUN(), 0.0f, 1.0f, 
														3*myFish.getRightFin().getUN(), 
														myFish.getRightFin().getVN(), myFish.getRightFin().getSideFinCoord());
			  		/*	Texture Mapping*/	
  			gl.glBindTexture(GL_TEXTURE_2D, texture[getFinTexture()]);
  			gl.glMap2f(GL_MAP2_TEXTURE_COORD_2, 0, 1, 2, 2, 0, 1, 4, 2, getTxtPoints());
  			gl.glEnable(GL_MAP2_TEXTURE_COORD_2);
      		gl.glEnable(GL_MAP2_VERTEX_3);
      		gl.glRotatef(180, 0.0f, 1.0f, 0.0f);
			gl.glMapGrid2f(20, 0.0f, 1.0f, 20, 0.0f, 1.0f);
  			gl.glEvalMesh2(GL_FILL, 0, 20, 0, 20);
		}
		
		/**
		*	setLight()
		*	Method that prepares and sets the light.
		*
		**/
		private void setLight() {
			float ambient[] = { 0.4f, 0.4f, 0.4f, 1.0f };
			float diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
			float position[] = { 1.0f, 1.0f, 1.0f, 0.0f };
			float lmodel_ambient[] = { 0.4f, 0.4f, 0.4f, 1.0f };
			gl.glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
			gl.glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
			gl.glLightfv(GL_LIGHT0, GL_POSITION, position);
			
			gl.glEnable(GL_LIGHT0);
			gl.glEnable(GL_LIGHTING);
		}
		
		/**
		*	loadGLTexture()
		*	Method that gets a String table as in parameter, and reads all the files stored in the table.
		*	@param files - String[] that holds all the files that are to be read.
		**/
		public void loadGLTexture(String[] files) {
			texture = new int[files.length];
			PngTextureLoader texLoader = new PngTextureLoader(gl, glu);
			gl.glGenTextures(files.length, texture);
			for(int i = 0; i < files.length; i++) {
				texLoader.readTexture(files[i]);
				if(texLoader.isOk()) {
					gl.glBindTexture(GL_TEXTURE_2D, texture[i]);
					gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
      				gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
					gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
      				gl.glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
      				gl.glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
					gl.glTexImage2D(GL_TEXTURE_2D,
					0,
					3,
					texLoader.getImageWidth(),
					texLoader.getImageHeight(),
					0,
					GL_RGB,
					GL_UNSIGNED_BYTE,
					texLoader.getTexture()
					);	
				}
			}	
		}
		
		/**
		*	makeFog()
		*	Method that makes the fog.
		*
		**/
		public void makeFog() {
			gl.glEnable(GL_FOG);
			float fogColor[] = { 0.5f, 0.6f, 1.0f, 1.0f };		
			
			gl.glFogfv(GL_FOG_COLOR, fogColor);
			gl.glFogf(GL_FOG_START, 30.0f);
			gl.glFogf(GL_FOG_END, 50.50f);
			gl.glFogi(GL_FOG_MODE, GL_LINEAR);	
		}
		
	/***************************************************************************************
	**
	**	newFish()
	**	Method that makes a new fish.
	**	@param factor - float that says how big the fish should be. Larger than 1.0 and the fish gets bigger, smaller than 1.0 and the fish gets smaller.
	**	@param type - String that says what type of fish that should be created. Only has an impact on the texture settings.
	********************************************************************************************/
		public void newFish(float factor, String type, String swimType) {
			if(type.equals("Default fish")) {
				setBodyTexture(0);
				setFinTexture(1);
				setEyeTexture(2);
			}
			if(type.equals("Blue fish")) {
				setBodyTexture(3);
				setFinTexture(4);
				setEyeTexture(5);
			}
			if(type.equals("Nice fish")) {
				setBodyTexture(6);
				setFinTexture(7);
				setEyeTexture(8);	
			}
			if(type.equals("Silver fish")) {
				setBodyTexture(9);
				setFinTexture(10);
				setEyeTexture(11);					
			}
			if(type.equals("Spotty fish")) {
				setBodyTexture(12);
				setFinTexture(7);
				setEyeTexture(13);				
			}
			if(swimType.equals("Default swim"))
				bezCurve = new BezierCurve(350);
			if(swimType.equals("Swim towards you"))
				bezCurve.changeBezier();
			myFish = new Fish(factor);	
		}
		
	/*********************************************************************************************
	**
	**	set and get methods for the variables.
	**
	*****************************************************************************************/	
		
		
		/**
		*	getTxtPoints()
		*	Method that returns the texture coordinates that are used in texture mapping.
		*	@return a float table holding the coordinates.
		**/
		private float[] getTxtPoints() {
			float txtpoints [] = {
       						-1.0f, -1.0f, 
       						 0.0f, -1.0f,
       						-1.0f, 0.0f, 
       						 0.0f, 0.0f
  								};
  			return txtpoints;
  		}
		 /**
		 *	setBodyTexture()
		 *	Method that sets the body texture.
		 *  @param filename - int that says which file is the valid one.
		 **/
		public void setBodyTexture(int filename) { bodyTexture = filename; }
		/**
		*	getBodyTexture()
		*	Method that returns the current bodyTexture.
		*	@return an int wich says which bodyTexture that is valid.
		**/
		public int getBodyTexture() { return bodyTexture; }
		/**
		*	setEyeTexture()
		*	Method that sets the eye texture.
		*	@param filename - int that says which position in the texture table is the valid eye texture.
		**/
		public void setEyeTexture(int filename) { eyeTexture = filename; }
		/**
		*	getEyeTexture()
		*	Method that returns the current eyeTexture
		*	@return the int that holds the position of the eyetexture in the texture table.
		***/
		public int getEyeTexture() { return eyeTexture; }
		/**
		*	setFinTexture()
		*	Method that sets the fin texture.
		*	@param filename - int that says which position in the texture table is the valid fin texture.
		**/		
		public void setFinTexture(int filename) { finTexture = filename; }
		/**
		*	getFinTexture()
		*	Method that returns the current finTexture
		*	@return the int that holds the position of the fintexture in the texture table.
		***/		
		public int getFinTexture() { return finTexture; }
		/**
		*	setFishAnimate() 
		*	@param b - boolean 
		**/
		public void setFishAnimate(boolean b) { fishAnimate = b; }
		/**
		*	setFishSwim()
		*	@param b - boolean
		**/
		public void setFishSwim(boolean b) { fishSwim = b; }
		/**
		*	setFogOn()
		*	@param b - boolean
		**/
		public void setFogOn(boolean b) { fogOn = b; }
		/**
		*	getFishAnimate()
		*	@return fishAnimate
		**/
		public boolean getFishAnimate() { return fishAnimate; }
		/**
		*	getFishSwim()
		*	@return fishSwim
		**/
		public boolean getFishSwim() { return fishSwim; }
		/**
		*	getFogOn()
		*	@return fogOn
		**/
		public boolean getFogOn() { return fogOn; }

		// get and set for the moving of the mouse
		/**
		*	General description for all the set and get methods.
		*
		*	set*()
		*	@param f - float that contains the new value.
		*
		*
		*	get*()
		*	@return float.
		**/
		public void setRotX(float f) { rotx = f; }
		public void setRotZ(float f) { rotz = f; }
		public float getRotX() { return rotx; }
		public float getRotZ() { return rotz; }
		public void setLastX(float f) { lastx = f; }
		public void setLastZ(float f) { lastz = f; }
		public float getLastX() { return lastx; }
		public float getLastZ() { return lastz; }
		public void setDeltaRotX(float f) { delta_rotx = f; }
		public void setDeltaRotZ(float f) { delta_rotz = f; }
		public float getDeltaRotX() { return delta_rotx; }
		public float getDeltaRotZ() { return delta_rotz; }
	}	