package fishProject;

import javax.swing.UIManager;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.lang.*;
import java.util.*;
import java.io.*;

/*******************************************************************************************
 * Title:        	FishWorld
 * Description:		The main class of the project. The class that controlls all the GUI.
 * @author 			Bjrn Hkon Horpestad
 * @version 		1.0
 *****************************************************************************************/

public class FishWorld extends JFrame implements ActionListener {
	
	// Panel variables	
	private JPanel contentPane;
	private JPanel fishPanel;

	// Variables to be used when making new fish
	private JTextField txtFactor;
	private JButton btnNewFish;
	private JLabel factorLbl;
	private JComboBox fishChoice;
	private JComboBox swimChoice;
	
	//Menu Bar variables
	private MenuBar menuBar;
   	private Menu file;
   	private Menu fishAction;
   	private Menu aboutMenu;
   	private MenuItem newFish;
   	private MenuItem exit;
   	private MenuItem animate;
   	private MenuItem stopAnimate;
   	private MenuItem swim;
   	private MenuItem stopSwim;
   	private MenuItem fog;
   	private MenuItem stopFog;
   	private MenuItem about;
	
	WindowHandler wHandler;
	MouseHandler mHandler;
	
	FishGLCanvas canv = null;
	
  /**
   * Constructor
   *	 
   * Sets up all the GUI things.	
   */
	public FishWorld() {
		contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		this.setSize(new Dimension(800, 500));
		this.setTitle("The Swimming Fish");	
		
		//This comes from GL4 java
		Dimension d = getSize();
		canv = new FishGLCanvas(d.width, d.height);
		contentPane.add("Center", canv);	
		
		//New fish panel
		swimChoice = new JComboBox();
		swimChoice.addItem("Default swim");
		swimChoice.addItem("Swim towards you");
		fishChoice = new JComboBox();
		fishChoice.addItem("Default fish");
		fishChoice.addItem("Blue fish");
		fishChoice.addItem("Nice fish");
		fishChoice.addItem("Silver fish");
		fishChoice.addItem("Spotty fish");
		btnNewFish = new JButton("Make new fish");
		btnNewFish.addActionListener(this);
		factorLbl = new JLabel("Fish size:");
		txtFactor = new JTextField(3);		
		fishPanel = new JPanel();
		fishPanel.add(swimChoice);
		fishPanel.add(fishChoice);
		fishPanel.add(factorLbl);
		fishPanel.add(txtFactor);
		fishPanel.add(btnNewFish);
		
		contentPane.add("South", fishPanel);
		fishPanel.setVisible(false);
		
		//Menu bar
		menuBar = new MenuBar();
      	setMenuBar(menuBar);
      
      	file = new Menu("File");
      	fishAction = new Menu("Fish Operations");
      	aboutMenu = new Menu("Help");
      	
      
      	menuBar.add(file);
      	menuBar.add(fishAction);
      	menuBar.add(aboutMenu);
      
      
      	newFish = new MenuItem("New fish");
      	file.add(newFish);
      	newFish.addActionListener(this);
      	
      	file.addSeparator();
      	
      	exit = new MenuItem("Exit");
	    file.add(exit);
      	exit.addActionListener(this);
      	
      	animate = new MenuItem("Start Animation");
      	fishAction.add(animate);
      	animate.addActionListener(this);
      	
      	stopAnimate = new MenuItem("Stop Animation");
      	fishAction.add(stopAnimate);
      	stopAnimate.setEnabled(false);
      	stopAnimate.addActionListener(this);      	
      	
      	fishAction.addSeparator();
      	
      	swim = new MenuItem("Start swimming");
      	fishAction.add(swim);
      	swim.addActionListener(this);
      	
      	stopSwim = new MenuItem("Stop swimming");
      	fishAction.add(stopSwim);
      	stopSwim.setEnabled(false);
      	stopSwim.addActionListener(this);
      	
      	fishAction.addSeparator();
      	
      	fog = new MenuItem("Make fog");
      	fishAction.add(fog);
      	fog.addActionListener(this);
      	
      	stopFog = new MenuItem("Quit the fog");
      	fishAction.add(stopFog);
      	stopFog.setEnabled(false);
      	stopFog.addActionListener(this);
      
      	about = new MenuItem("About");
      	aboutMenu.add(about);
      	//about.addActionListener(this);
      
	
		// MouseListener
		mHandler = new MouseHandler();
		canv.addMouseListener(mHandler);
		canv.addMouseMotionListener(mHandler);
		
		// WindowListener
		wHandler = new WindowHandler();
		this.addWindowListener(wHandler);
		
		pack();
		show();
		
		canv.start();
	}
	/**
	*	actionPerformed()
	*	This method takes care of the actionEvents.
	*	@param e - an ActionEvent, that decides what to be done.
	*
	*/
	
  	public void actionPerformed(ActionEvent e){
  		if(e.getActionCommand() == "Make new fish") {
  			String tmp2 = (String)swimChoice.getSelectedItem();
  			String tmp = (String)fishChoice.getSelectedItem();
  			if(txtFactor.getText().equals(""))
  				canv.newFish(1.0f, tmp, tmp2);
  			else 
  				canv.newFish(Float.parseFloat(txtFactor.getText()), tmp, tmp2);
  			fishPanel.setVisible(false);
  		}
  		if(e.getActionCommand() == "New fish")
  			fishPanel.setVisible(true);
  		if(e.getActionCommand() == "Start Animation"){
  			canv.setFishAnimate(true);	
  			animate.setEnabled(false);
  			stopAnimate.setEnabled(true);
  		}
  		if(e.getActionCommand() == "Stop Animation") {
  			canv.setFishAnimate(false);
  			animate.setEnabled(true);
  			stopAnimate.setEnabled(false);	
  		}
  		if(e.getActionCommand() == "Start swimming") {
  			canv.setFishSwim(true);	
  			swim.setEnabled(false);
  			stopSwim.setEnabled(true);
  		}
  		if(e.getActionCommand() == "Stop swimming") {
  			canv.setFishSwim(false);
  			swim.setEnabled(true);
  			stopSwim.setEnabled(false);	
  		}
  		if(e.getActionCommand() == "Make fog") {
  			canv.setFogOn(true);
  			fog.setEnabled(false);
  			stopFog.setEnabled(true);
  		}
  		if(e.getActionCommand()  == "Quit the fog") {
  			canv.setFogOn(false);
  			fog.setEnabled(true);
  			stopFog.setEnabled(false);	
  		}
  		if(e.getActionCommand() == "Exit")
  			System.exit(0);
  	}
	
	/**
	*	main()
	*	The main method that starts everything.
	*	@param args[] - a table of arguments.
	*
	**/
	public static void main(String[] args) { new FishWorld(); }
	
	/******************************************************
	** Inner window handler class
	**
	******************************************************/
	class WindowHandler implements WindowListener {
		public void windowOpened(WindowEvent we) {}
		/**
		*	windowClosing()
		*	This method closes the program
		*	@param we - WindowEvent
		**/
		public void windowClosing(WindowEvent we) { System.exit(0); }
		public void windowClosed(WindowEvent we) {}
		public void windowIconified(WindowEvent we) {}	
		public void windowDeiconified(WindowEvent we) {}
		public void windowActivated(WindowEvent we) {}
		public void windowDeactivated(WindowEvent we) {}
	}
	
	
	/*************************************************************
	**
	** Inner mouse handler class
	**
	**************************************************************/
	class MouseHandler implements MouseListener, MouseMotionListener {
		public void mouseEntered(MouseEvent e) {}	
		public void mouseExited(MouseEvent e) {}
		public void mouseMoved(MouseEvent e) {}
		public void mouseClicked(MouseEvent e) {}
		public void mouseReleased(MouseEvent e) {}
		/**
		*	mousePressed()
		*	Method that stores the values of x and y when mouse is pressed.
		*	@param e - MouseEvent, the mouseEvent that has been done.
		**/
		public void mousePressed(MouseEvent e) {
			canv.setLastX(e.getX());
			canv.setLastZ(e.getY());
		}
		/**
		*	mouseDragged()
		*	Method that registers the movement made by the mouse when dragged on the screen. 
		*	The changes are implied on the fish in the canvas.
		*	@param e - MouseEvent, the mouseevent that has been done.
		*
		**/
		public void mouseDragged(MouseEvent e) {
			int x = e.getX();
			int y = e.getY();
			Dimension dim = e.getComponent().getSize();
			int width = dim.width;
			int height = dim.height;
			
			float tmp1 = canv.getLastX();
			float tmp2 = canv.getLastZ();
			
			canv.setDeltaRotZ((90.0f*((float)(x-tmp1)/(float)(width))));
			canv.setDeltaRotX((90.0f*((float)(y - tmp2)/(float)(height))));
			
			tmp1 = canv.getRotZ();
			tmp2 = canv.getDeltaRotZ();
			canv.setRotZ(tmp1 + tmp2);
			tmp1 = canv.getRotX();
			tmp2 = canv.getDeltaRotX();
			canv.setRotX(tmp1 + tmp2);
			
			canv.setLastX(x);
			canv.setLastZ(y);
			
			canv.repaint();	
		}		
	}
	
}

