package fishProject;

/*********************************************************************
 * Title:      	Tail
 * Description:	Class to represent the fish tail.
 * @author 		Bjrn Hkon Horpestad
 * @version 	1.0
*******************************************************************/

public class Tail {
	private float[] tailCoord;
	private int un = 2;
	private int vn = 4;
	private float factor = 1.0f;
	private float zStart1, zStart2;
	private float zCtrp1_1, zCtrp2_1, zCtrp1_2, zCtrp2_2;
	private float zEnd1, zEnd2;
	
	/**
	*	Tail()
	*	Constructor
	**/
	public Tail() {
		setTailCoord();
	}
	
	/**
	*	Tail()
	*	@param fac - float that represents the size of the tail
	**/
	public Tail(float fac) {
		factor = fac;
		setCoordinates();
		setTailCoord();	
	}
	
	/**
	*	setCoordinates()
	*	Method that sets the coordinates which are changed because of the animation
	**/
	private void setCoordinates() {
		zStart1 = 0.0f* factor; zCtrp1_1 = 0.0f * factor; zCtrp2_1 = 0.0f * factor; zEnd1 = 0.0f * factor;
		zStart2 = 0.0f * factor; zCtrp1_2 = 0.0f * factor; zCtrp2_2 = 0.0f * factor; zEnd2 = 0.0f * factor;
	}
	
	/**
	*	setTailCoord()
	*	Method that sets all the control points for the bezier surface.
	**/
	public void setTailCoord() {
		float tmpTailCoord[] = {
							-4.5f* factor, 1.2f* factor, zStart1,
							-3.3f* factor, 0.1f* factor, zCtrp1_1,
							-4.5f* factor, 1.4f* factor, zCtrp2_1,
							-3.3f* factor, 0.2f* factor, zEnd1
							,
							-4.5f * factor, -1.2f* factor, zStart2,
							-3.3f * factor, 0.0f * factor, zCtrp1_2,
							-4.5f * factor, -1.4f * factor, zCtrp2_2,							
							-3.3f * factor, -0.1f * factor, zEnd2
						};		
		tailCoord = tmpTailCoord;
	}
	
	/**
	*	getTailCoord()
	*	@return float[] table containing the control points for the bezier surface
	**/
	public float[] getTailCoord() { return tailCoord; }
	
	/**
	*	getUN()
	*	@return un
	**/
	public int getUN() { return un; }
	/**
	*	getVN()
	*	return vn
	**/
	public int getVN() { return vn; }
		
	/**
	*	moveTail()
	*	Method that moves all the z-coordinates for the tail, so that they correspond with newZ
	*	@param newZ - float
	**/	
	public void moveTail(float newZ) {
		zStart1 = newZ; zCtrp1_1 = newZ; zCtrp2_1 = newZ; zEnd1 = newZ;
		zStart2 = newZ;	zCtrp1_2 = newZ; zCtrp2_2 = newZ; zEnd2 = newZ;
		setTailCoord();
	}
}